/******************************************************************************/
/*          ______  ___   ______  ___   __________   ______                   */
/*          ___   |/  /   ___   |/  /   ___  ____/   __<  /                   */
/*          __  /|_/ /    __  /|_/ /    __  /_       __  /                    */
/*          _  /  / /     _  /  / /     _  __/       _  /                     */
/*          /_/  /_/      /_/  /_/      /_/          /_/                      */
/*                                                                (2009/2010) */
/******************************************************************************/
/*  Program rjesava jednodimenzionalne odredejne integrale pomocu Simpsonovog */
/*  i Trapeznog pravila te sprema rezultate u pripadne datoteke.              */
/*  Potrebno je mijenjati:                                                    */
/*    - donju granicu integracije  (x1)                                       */
/*    - gornju granicu integracije (x2)                                       */
/*    - maksimalni broj podijela segmenta preko kojeg integriramo (max_in)    */
/*    - vrijednost dobivenu analiticki ako racunamo relativnu pogresku (exact)*/
/*    - podintegralnu funkciju f(x)                                           */
/*                                                                            */
/******************************************************************************/

#include<stdio.h>     // Deinirane standardne ulazno, izlazne funkcije
#include<stdlib.h>    // Deinirana nama potrebna funkcija za pauziranje
#include <math.h>     // Definirane potrebne matematicke funkcije
 
#define max_in  1001  // Maksimalni broj podijela (neparan broj zbog Simpsona)
#define x1      0.1   // Donja granica integracije
#define x2      0.2   // Gornja granica integracije
#define exact   0.45  // Vrijednost dobivena analiticki

int main(void){
  int i;
  double i_t,i_s,rg_t,rg_s;// Numericke vrijednosti integrala i relativne greske
  double f(double);        // Podintegralna funkcija
  FILE *fpt, *fps;         // Datoteke za pohranu numerickih vrijednosti
  double trapez  (int ndx, double min, double max);    // Trapezno pravilo
  double simpson (int ndx, double min, double max);    // Simpsonovo pravilo

  /* Otvaranje potrebnih datoteka */
  if ((fpt = fopen("trapez.txt","w")) == NULL) {
    printf("Nije moguce pronaci izlaznu datoteku\n");
    system("PAUSE");
    return 0;
  }
  if ((fps = fopen("simpson.txt","w")) == NULL) {
    printf("Nije moguce pronaci izlaznu datoteku\n");
    system("PAUSE");
    return 0;
  }
  fprintf(fpt,"%52s\n","#          N              Trapez    Relativna greska");
  fprintf(fpt,"%52s\n","#  ---------   -----------------   -----------------");
  fprintf(fps,"%52s\n","#          N             Simpson    Relativna greska");
  fprintf(fps,"%52s\n","#  ---------   -----------------   -----------------");

  /*Pozivanje funkcija za numericku integraciju */
  for (i=3; i<=max_in; i+=4){   //Pocetni broj mora biti neparan zbog Simpsona
    i_t = trapez(i, x1, x2);
    rg_t = fabs((i_t-exact)/exact);
    fprintf(fpt, "%12i%20.9f%20.9f\n", i, i_t, rg_t);
    i_s = simpson(i, x1, x2);
    rg_s = fabs((i_s-exact)/exact);
    fprintf(fps, "%12i%20.9f%20.9f\n", i, i_s, rg_s);
  }
  printf("\n*************************************************************");
  printf("\n* Numericka rjesenja pohranjena su u izlaznim datotekama!\n");
  printf("\n*************************************************************\n");
  fclose(fpt);
  system("PAUSE");
  return 0;
}

/* Definicija podintegralne funkcije */
double f (double x){ 
   return (30.*x);               
}

/* Trapezno pravilo za numericku integraciju */
double trapez (int ndx, double min, double max){
  int i;
  double dx, sum=0., x;
  dx = ((max-min) / (ndx-1));
  // Doprinos unutarnjih tocaka
  for (i=2; i<ndx; i++) {
    x    = min + dx * (i-1);
    sum += f(x)*dx;
  }
  // Doprinos krajnjih tocaka
  sum += 0.5 *(f(min) + f(max)) * dx;
  return (sum);
}      

/* Simpsonovo pravilo za numericku integraciju*/
double simpson (int ndx, double min, double max){  
  int i;
  double dx, sum=0., x;
  dx = ((max -min) /(ndx-1));
  // Vrijednosti u neparnim tockama
  for (i=2; i<ndx; i+=2) {
    x = min + dx * (i-1);
    sum += 4 * f(x);
  }
  // Vrijednosti u parnim tockama
  for (i=3; i<ndx; i+=2) {
    x = min + dx * (i-1);
    sum += 2 * f(x);
  }
  // Vrijednosti u krajnjim tockama
  sum +=  f(min) + f(max);
  // Suma vrijednosti podijeljena sa 3 i pomnozena sa duljinom podsegmenta 
  sum *= dx/3.;
  return (sum);
}  
